/***************************************************************
*** File: imageio.h
*** Author: John Y. A. Wang
*** Date: March 10, 1994.
*** Copyright 1994, Massachusetts Institute of Technology, All Rights Reserved.
***
*** Description: header for image IO commands
***************************************************************/

#ifndef IMAGEIO_INCLUDED
#define IMAGEIO_INCLUDED

#include "motion-sh.h"

static  char *copyright_io="\
\n\n***********************************************\n\
File: imageio.h\n\
Author: John Y. A. Wang.\n\
The MIT Media Lab.\n\
Copyright (c) 1994, All Rights Reserved.\n\
***********************************************\n\n";

/* datatype formats */
#define IM_UCHAR 1
#define IM_FLOAT 2

/***** conversions ******/
#define F2UC(f) ((unsigned char)((f>255)?255:((f<0)?0:f)))
#define RGB2Y_R 0.229
#define RGB2Y_G 0.587
#define RGB2Y_B 0.114


/* filetype formats */
#define RAW      0
#define STREAM   1
#define DATASET  2
#define DATAFILE 3


/* structure for accessing disk. */
/* typestr[0] for filetype specification:
 *   'f'=datafile; 's'=dataset; 'r'=rawfile; 'i'=stream;
 * typestr[1] for datatype specification:
 *   'b'=byte(unsigned char); 'f'=float
 */
struct Dataset {
  FILE *fp;
  char *typestr;  /* ff fb sf sb rf rb if ib */
  int filetype;
  int datatype;
  int elmsize;
  int headersize;
  char *filename;
  int nchannels;
  int xdim;
  int ydim;
  int imsize;
  int start;
  int end;
  int cur_frame;
  int cur_channel;
};
typedef struct Dataset Dataset;

struct Image {
  int xdim;
  int ydim;
  int nchannels;
  int imsize;
  int datatype;
  int elmsize;
  unsigned char *data;
};
typedef struct Image Image;

#endif

/****** Multi channel selection and conversion options for get_im() ********/
/* the ch argument in get_im() can take on the following: */
#define ALL_CHANNELS -1        /* select all the channels */
#define RGB2Y        -2        /* convert from RGB format to Y. 3ch -> 1ch */
/* values specified in the range [0...nchannels-1] indicates to select
 * that particular channel */

/********************** prototypes *****************************/

/* create a single image with memory for one channel */
/* calls create_im_mc to allocate the memory for one channel */
/* for back-compatibility */
extern int create_im(/*im_ptr,datatype,xdim,ydim*/);

/* allocate memory image structure and image data for multiple channels */
extern int create_im_mc(/*im_ptr,datatype,xdim,ydim,nchannels*/);

/* free image structure memory. returns 0 on success */
extern int free_im(/*im_ptr*/);

/* get frame from IO: stream, dataset, datafile, or raw. */
/* returns 1 on success. returns 0 on end of file. returns -1 on failure.  */
extern int get_image(/*ds,frame,ch,im*/);

/* initializes dataset structure */
extern int init_ds(/*ds*/);

/* close up the dataset. need to close any opened streams */
extern int close_ds(/*ds*/);

/* verify that the elements in dataset struct is consistant and valid */
/* also set the filetype and datatype from typestr */
/* returns 0 when valid */
extern int verify_ds(/*ds*/);

/* verifies that the datatype is valid. Return the number of
 * bytes associated with the desired output data type. 
 * Currently only checks for data type and allows for output stream
 * and raw datatype
 */
extern int verify_outtype(/*filename,outtype*/);

/* show the contents of the dataset structure ******/
extern int show_ds(/*ds*/);

