/***************************************************************
*** File: rgb2y.c
*** Author: John Y. A. Wang
*** Date: January 28, 1993.
*** Copyright 1993, Massachusetts Institute of Technology, All Rights Reserved.
***
*** Description: binary image operations
***************************************************************/

static  char *copyright="\
\n\n***********************************************\n\
File: rgb2y.c\n\
Author: John Y. A. Wang.\n\
The MIT Media Lab.\n\
Copyright (c) 1993, All Rights Reserved.\n\
***********************************************\n\n";

#include "imageio.h"

#ifdef lint
#define register
#endif

#define EXIT  { \
     close_ds(ds1); free_im(&im1_s);\
     close_files(opened_files,nfiles); return;}

main(argc,argv)
  int argc;
  char **argv;
{
  int i=0,j=0,c=0,r=0;
  int i1;
  int num=0;

  int verbose=0;

  char *outfile=NULL;
  FILE *ofp=NULL;

  Image *im1_s=NULL;
  Dataset dataset1,*ds1= &dataset1;
  unsigned char *im=NULL;

  int xdim= -1, ydim= -1,imsize;
  int nchannels=1;
  int start=0,end=1000;

  int destroy=0;


  char *cmd="\
rgb2y -xdim -ydim [-i filename] [-intype {s,f,r,i | b,f}] \n\
     -o (float)<outfile> [-destroy] [-v]"; 


  FILE *opened_files[MAXNFILES];
  int nfiles=0;
  int cimsize;

  char *progn=argv[0];

  /********** begin ***********/
  init_ds(ds1);

  for(i=0,nfiles=0;i<MAXNFILES;i++) { opened_files[i]=NULL;  }

  /********************* begin scan input ************************/
  if(argc==1) { fprintf(stderr,"\n%s\n\n",cmd); return;}
  BEGIN_SCAN(argc,argv,i,j,cmd) {
    scanarg1(argc,argv,i,"-xdim","%d",&xdim,-1);
    scanarg1(argc,argv,i,"-ydim","%d",&ydim,-1);

    setflag1(argc,argv,i,"-destroy",destroy,1,0);
    setflag1(argc,argv,i,"-v",verbose,1,0);

    setarg1(argc,argv,i,"-o",outfile,NULL);
    setarg1(argc,argv,i,"-i",ds1->filename,NULL);
    setarg1(argc,argv,i,"-intype",ds1->typestr,NULL); 
  }
  END_SCAN(argc,argv,i,j,cmd);

  /************************ end scan input ***************************/

  nchannels=3;
  
  if(xdim<=0 || ydim<=0) {
    fprintf(stderr,
	    "%s: incorrect input dimensions specification: xdim=%d, ydim=%d\n",
	    progn,xdim,ydim); return;
  }
  
  imsize=xdim*ydim;


  ds1->xdim=xdim; ds1->ydim=ydim*nchannels;;  ds1->nchannels=1;
  cimsize=ds1->xdim*ds1->ydim;

  if(end<start) {
    fprintf(stderr,"end=%d less than start=%d. No frames selected\n",end,start);
    fflush(stderr);return;
  }
    


  /************** set files ****************/
  
  /* default is stream or raw float input */
  if(ds1->typestr==NULL) {
    if(ds1->filename==NULL) { ds1->typestr="ib"; }
    else { ds1->typestr="rb"; }}

  if(verify_ds(ds1)!=0) {
    fprintf(stderr,"incorrect file1 specification\n"); EXIT;}

  /****** memory allocation ********/

  create_im(&im1_s,IM_UCHAR,ds1->xdim,ds1->ydim);
  im=(unsigned char *)im1_s->data;

  /***********************************************/

  for(i1=ds1->start;i1<=ds1->end;i1++) {
    if((num=get_image(ds1,i1,0,im1_s))!=1) { EXIT; }
    
/*    fprintf(stderr,"i=%d\n",i1); */
    for(i=0,r=0;i<imsize;i++,r+=3) {
      im[i]=0.299*im[r]+0.587*im[r+1]+0.114*im[r+2];
    }

/*    fprintf(stderr,"done conversion\n"); */

    if(i1==ds1->start || destroy==1) {
/*      fprintf(stderr,"reopening file...\n"); */
      if((ofp=set_out_file(outfile)) == NULL)  {EXIT;}
      else { opened_files[nfiles]=ofp; nfiles++; }
    }
      
/*    fprintf(stderr,"outputing file.\n"); */

    if((num=fwrite(im,sizeof(unsigned char),imsize,ofp))!=imsize) {
      fprintf(stderr,"incomplete write on frame %d:%d. %d of %d written.\n",
	      i1,c,num,imsize); EXIT;
    }
    fflush(ofp);
    if(destroy==1) { close_files(opened_files,nfiles); nfiles=0;}
    if(ofp!=stdout && verbose==1) {
      fprintf(stderr,"rgb2y: Done saving to file\n");
    }
  }
    /********** clean up *********/
  fprintf(stderr,"\n");fflush(stderr);
  EXIT;
}
