/*---------------------------------------------------------------------------

  Program Name:  defines.h
  Purpose:       The defines for the face processing pipeline.
                 
  Author:        David Becker
  Date:          Sept. 21, 1994

-----------------------------------------------------------------------------*/
/*** CONSTANT DEFINES ***/

/* Define flags for the search procedures */

/* There are some constants that have a second version with a '2' */
/* (e.g. DO_GRAYMAP and DO_GRAYMAP2).  The one without the '2' is  */
/* for ms_eigsearch and the one with the 2 is for eigsearch.      */

#define DO_GRAYMAP           0 

#define DO_UNITMAP           1

#define DO_MAHALANOBIS       1

#define DO_SPATIAL           1       /* use spatial priors? */

#define EIG_FIRST            1       /* number of eigenvectors to */
#define EIG_LAST             10      /* use for headfinding */

#define DO_GRAYMAP2          0

#define DO_UNITMAP2          1

#define DO_MINIMA            1

#define DO_MAHALANOBIS2      1

#define DO_SPATIAL2          1

#define DO_WINDOWS           1

#define MULTIPLE_HEADSEARCH  0        /* whether to search for */
				      /* multiple heads */

#define EIG_FIRST2           1        /* number of eigenvectors to use */
#define EIG_LAST2            5        /* for feature finding */

#define NUM_COEFFS           100      /* num of eigenvectors to use in */
				      /* projections */

#define NUM_OF_NEIGHBORS     3        /* Number of people to return as */
				      /* nearest neighbors to given */
				      /* image */

#define SCALEWARP_NCOL       128      /* pixel size of images AFTER */
#define SCALEWARP_NROW       128      /* being processed */

#define MS_EIGSEARCH_NROW    192      /* pixel size of images going */
#define MS_EIGSEARCH_NCOL    128      /* in to be processed */

#define DO_RIGID             1        /* do rigid (1) or affine (0) */
				      /* warping */

#define DO_RIGID_PT1         1        /* features to use in rigid */
#define DO_RIGID_PT2         2	      /* warping (1,2 are leye, reye) */

#define CONTRAST_SCALE       70
#define CONTRAST_PEDESTAL    100

#define MEAN_SCALE           1.0      /* expected scale of feature */
				      /* constellation for ml_detect */

#define TOP_N_MAX            10       /* # of local minima for each */
				      /* feature in ml_detect */

#define DO_READ_MINIMA       1        /* flag for whether to read */
				      /* minima files rather than */
				      /* distance maps in ml_detect */

#define SCALEFACTOR          4.7      /* scale factor for scalewarp */

#define BGVALUE              237.0    /* pixel value in void regions */
				      /* for scalewarp */

#define X_DEST               59.0     /* destination row position for */
				      /* head center in warped image */

#define Y_DEST               72.0     /* destination col position for */
				      /* head center in warped image */

#define NONZERO_FLAG         1        /* flag controlling whether or */
				      /* not to only use nonzero */
				      /* pixels in computing contrast */

/* data files */
#define SCALEFILE           "fab_headscales_extended.dat"
#define MASKFILE            "pipeline_mask.bf"

#define FLT_MYMAX            1e38

#ifndef FALSE
#define FALSE		     (0)
#endif
#ifndef TRUE
#define TRUE		     (!FALSE)
#endif

/* ms_search constants */
#define DO_VAR                  0       /* variance check */
#define VAR_THRESHOLD           100     /* threshold for low-var patches */
#define VAR_MAXVALUE            1e7     /* replacement in distance map   */

#define MAX_NUM_TEMPLATES       5
#define MAX_NUM_SCALES          50
#define MAX_NUM_EIGENVECTORS    128     /* for static storage in dffs() */
#define MAX_VECTOR_LENGTH       16      /* "    " in mahalanobis_dist() */
#define MAX_CHARS               256

#define MAX_NUM_MLTEMPLATEs     6
#define MAX_NUM_MINIMA          100
#define SCALEX                  59
#define SCALEY                  72

/*---------------------------------------------------------------------*/

/*** STRUCTURES / TYPEDEFS ***/

/* Other typedefs... */

#ifndef boolean
typedef int             boolean;
#endif

/* Image types ... */

typedef float	        *image_line_t;	/* a single dynamic image scanline */
typedef image_line_t	*image_t;	/* dynamic 2D image */

/*---------------------------------------------------------------------*/

/* Image file structure definitions */

typedef struct s_srchparms
{
  int nfeatures;                /* Number of features to search for.    */
  float **model;
  float **geo_M;
  int **templatesize;           /* Size of the template array           */
  float ***template; 
  int *N_DIM;                   /* the dimensionality of each template  */
  int *M_DIM;                   /* # of eigenvectors for each template  */
  int sets;                     /* sets from the descriptor file        */
  int bytes_pixel;              /* bytes per pixel from descriptor file */
  int ncol;                     /* # of cols of image                   */
  int nrow;                     /* # of rows of image                   */
  int geo_nrow;
  int geo_ncol;
  int numscales;                /* number of scales in the scale file   */
  int max_N;                    /* the max num of N                     */
                  /* Old Command Line Defaults */
  int do_graymap;               /* graymap each patch?                  */
  int do_unitmap;               /* unitmap each patch?                  */
  int do_mahalanobis;           /* extra computation?                   */
  int do_spatial;               /* add [x,y] Mahalanobis dist?          */
  int do_windows;               /* limit search to a window?            */
  int do_minima;                /* output minima files?                 */
  int multiple_search;          /* search for multiple heads?           */
  int eig_first;                /* which eigenvector to start on?       */
  int eig_last;                 /* which eigenvector to end on?         */
                 /* End Old Command Line Defaults */
  float *scales;                /* vector of the scales for the search  */
  image_t orig_image;           /* the U_char image grabbed by camera   */
} srchparams_t;

typedef struct s_imagparams
{
  /* Specialized for the case of a one-feature search */
  int   lefteye_x;
  int   lefteye_y;
  float lefteye_e;
  int   righteye_x;
  int   righteye_y;
  float righteye_e;
  int   nose_x;
  int   nose_y;
  float nose_e;
  int   mouth_x;
  int   mouth_y;
  float mouth_e;
  int   best_head_x;
  int   best_head_y;
  float best_head_e;
  float best_scale;            
  int   second_head_x;
  int   second_head_y;
  float second_head_e;
  float second_scale;
  int   third_head_x;
  int   third_head_y;
  float third_head_e;
  float third_scale;
  float energy;

} imagparams_t;
/*---------------------------------------------------------------------*/

/* Typedefs used in pipe_neighbor.c; these are structures to hold */
/* information about faces and the coefficients that go with them */

typedef struct match *Matchptr;

typedef struct match{
  char *name;
  float distance;
  Matchptr next;
} Match;

typedef struct kface{
  char *name;
  float *coeffs;
} Kface;

typedef struct tface {
  char *name;
  Matchptr similar;
} Tface;

/* --- Stuff from match.c --- */

struct Points {
  int n;
  int *x;
  int *y;
  float *f;
};

struct Point {
  int x;
  int y;
  float f;
};

/*---------------------VIDEO STUFF--------------------------------*/

/* DIRECTORIES FOR ADDING IMAGES TO DIGITIAL EXPRESSIONS DATABASE */

#define PHOTOBOOK_ORIG "/mas/vision/bigu/dbecker/de/photobook/data/feret/image-orig/"
#define PHOTOBOOK_NORM "/mas/vision/bigu/dbecker/de/photobook/data/feret/image-norm/"
#define PHOTOBOOK_HEAD "/mas/vision/bigu/dbecker/de/photobook/data/feret/image-head/"
#define PHOTOBOOK_COEFF "/mas/vision/bigu/dbecker/de/photobook/data/feret/picture-ev/"
#define COLOR_DIR "/mas/vision/bigu/dbecker/de/colorimages/"


#define VID_CUT_ROW 480         /* rows and columns to cut out of */
#define VID_CUT_COL 480		/* digitizing video image*/

#define VID_ROW 480             /* rows and columns in video image. */
#define VID_COL 640		/* COL MUST BE > ROW */

#define GPFILTER 0.4            /* Kind of filter to decimate video */
				/* image with */

/*-----------------------Hiererachical Stuff----------------------------*/

#define HIER_BOX_SIZE 3         /* Only one minimum from cubes with */
				/* this size edge are checked for */
				/* features. */

#define HIER_TOP_N 5            /* The number of head candidates to */
				/* check in the hierarchical head */
				/* detection */
