/***************************************************************
*** File: display-digit.c
*** Author: John Y. A. Wang
*** Date: January 25, 1992.
*** Copyright 1992, Massachusetts Institute of Technology, All Rights Reserved.
***
*** Description:
***************************************************************/


#ifndef DISPLAY_NUMBER_INCLUDED
#define DISPLAY_NUMBER_INCLUDED
/*************** these thing need do only once *************/
#include<stdio.h>
#include "motion.h"


static  char *copyright_dsp_num_h="\
\n\n***********************************************\n\
File: display-number.h\n\
Author: John Y. A. Wang.\n\
The MIT Media Lab.\n\
Copyright (c) 1992, All Rights Reserved.\n\
***********************************************\n\n";

#define inbounds(x,dim)  ( ((x)>=0) && ((x)<(dim)) )

/*************** prototypes ************************/
extern void display_number_float_im();
static void display_digit_bitmap_float();

extern void display_number_uchar_im();
static void display_digit_bitmap_uchar();

/**** these are for compatibility with display-digit.c *****/
extern void display_number_on_image();
#define display_float_on_image display_number_uchar_im

#endif   /* do only once stuff */


/****************************************************************/
/* build the display number routines for particular 
 * data types here.
 * define BUILD_DISPLAY_NUMBER to build the functions
 * #define JIMAGE_TYPE_SWITCH 'f' for float operations
 * #define JIMAGE_TYPE_SWITCH 'u' for unsigned char  operations
 */

#ifdef BUILD_DISPLAY_NUMBER

#if (JIMAGE_TYPE_SWITCH == 'f')
#  define IMAGE_TYPE float
#  define display_number_im display_number_float_im
#  define display_digit_bitmap display_digit_bitmap_float

#else
#  if (JIMAGE_TYPE_SWITCH == 'u')
#    define IMAGE_TYPE unsigned char
#    define display_number_im display_number_uchar_im
#    define display_digit_bitmap display_digit_bitmap_uchar
#  endif
#endif

#ifndef NUMBER_BITMAP_INCLUDED
#define NUMBER_BITMAP_INCLUDED
/* if building number then include the static pictures, but only once. */
/***********  static pictures of numbers *******************/
/* bitmaps of numbers: generated by Unix "bitmap" program */

#define zero_width 5
#define zero_height 5
static char zero_bits[] = {
   0x0e, 0x0a, 0x0a, 0x0a, 0x0e};
#define one_width 5
#define one_height 5
static char one_bits[] = {
   0x04, 0x04, 0x04, 0x04, 0x04};
#define two_width 5
#define two_height 5
static char two_bits[] = {
   0x0e, 0x08, 0x0c, 0x02, 0x0e};
#define three_width 5
#define three_height 5
static char three_bits[] = {
   0x0e, 0x08, 0x0e, 0x08, 0x0e};
#define four_width 5
#define four_height 5
static char four_bits[] = {
   0x08, 0x0c, 0x0a, 0x0e, 0x08};
#define five_width 5
#define five_height 5
static char five_bits[] = {
   0x0e, 0x02, 0x0e, 0x08, 0x0e};
#define six_width 5
#define six_height 5
static char six_bits[] = {
   0x02, 0x02, 0x0e, 0x0a, 0x0e};
#define seven_width 5
#define seven_height 5
static char seven_bits[] = {
   0x0e, 0x08, 0x04, 0x04, 0x04};
#define eight_width 5
#define eight_height 5
static char eight_bits[] = {
   0x0e, 0x0a, 0x0e, 0x0a, 0x0e};
#define nine_width 5
#define nine_height 5
static char nine_bits[] = {
   0x0e, 0x0a, 0x0e, 0x08, 0x08};
#define dot_width 5
#define dot_height 5
static char dot_bits[] = {
   0x00, 0x00, 0x00, 0x04, 0x04};
/****************************************************************/
#endif   


/* display the number as digits on an image. */
extern void display_number_im(array,xdim,ydim,number,
				   xoffset,yoffset,fg_val,bg_val,format)
  IMAGE_TYPE *array;
  int xdim,ydim;
  float number;
  int xoffset,yoffset;
  int fg_val,bg_val;
  char *format;
{
  char alpha[10];
  int len,i,offset;
  int prev_xoffset;

/*  ndigits=2; */
  char buf[10];
  int f_last=strlen(format)-1;

/*  fprintf(stderr,"f_last=%d format[f_last]='%c'\n",f_last,format[f_last]); */
  if(format[f_last]=='d') {
    sprintf(alpha,format,(int)number); }
  else if(format[f_last]=='f') {
    sprintf(alpha,format,(float)number); }
  else {
    fprintf(stderr,"display_number: invalid format string \"%s\"\n",format);
    fprintf(stderr,"alpha=\"%s\":",alpha);
  }

  len=strlen(alpha);

  offset=xoffset; prev_xoffset=0;

  for(i=0;i<len;i++) {
    offset=offset+prev_xoffset;
    switch(alpha[i]) {
    default:
    case '0':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   zero_bits,zero_width,zero_height,fg_val,bg_val);
      prev_xoffset=zero_width; break;
    case '1':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   one_bits,one_width,one_height,fg_val,bg_val);
      prev_xoffset=one_width; break;
    case '2':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   two_bits,two_width,two_height,fg_val,bg_val);
      prev_xoffset=two_width; break;
    case '3':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   three_bits,three_width,three_height,fg_val,bg_val);
      prev_xoffset=three_width; break;
    case '4':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   four_bits,four_width,four_height,fg_val,bg_val);
      prev_xoffset=four_width; break;
    case '5':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   five_bits,five_width,five_height,fg_val,bg_val);
      prev_xoffset=five_width; break;
    case '6':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   six_bits,six_width,six_height,fg_val,bg_val);
      prev_xoffset=six_width; break;
    case '7':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   seven_bits,seven_width,seven_height,fg_val,bg_val);
      prev_xoffset=seven_width; break;
    case '8':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   eight_bits,eight_width,eight_height,fg_val,bg_val);
      prev_xoffset=eight_width; break;
    case '9':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   nine_bits,nine_width,nine_height,fg_val,bg_val);
      prev_xoffset=nine_width; break;
    case '.':
      display_digit_bitmap(array,xdim,ydim,offset,yoffset,
			   dot_bits,dot_width,dot_height,fg_val,bg_val);
      prev_xoffset=dot_width;
      break;
    }
  }
}


static void display_digit_bitmap(array,xdim,ydim,xoffset,yoffset,
				digit,dxdim,dydim,fg_val,bg_val)
  IMAGE_TYPE *array;
  int xdim,ydim,xoffset,yoffset;
  int dxdim,dydim;
  char *digit;
  int fg_val,bg_val;
{
  int x,y,ix,iy;
  int yhash=(int)((dxdim-1)/8 + 1);

/*
  printf("display_digit_bitmap: (%d, %d) (%d, %d) (%d, %d) fg_val=%d, bg_val=%d, yhash=%d\n",
	 xdim,ydim,xoffset,yoffset,dxdim,dydim,fg_val,bg_val,yhash);
*/
  
  for(iy=0,y=yoffset; iy<dydim ;y++,iy++) {
    for(ix=0,x=xoffset; ix<dxdim ;x++,ix++) {
      if(inbounds(x,xdim) && inbounds(y,ydim))
	if( ((digit[(iy*yhash)+(int)(ix/8)]) >> (ix-(int)(ix/8))) & 0x0001 )
	  array[y*xdim+x] = (IMAGE_TYPE)fg_val;
	else
	  array[y*xdim+x] = (IMAGE_TYPE)bg_val;
    }
  }
}

#endif   /* BUILD_DISPLAY_NUMBER */

#undef IMAGE_TYPE
#undef display_number_im
#undef display_digit_bitmap
