/***************************************************************
*** File: motion-sh.h
*** Author: John Y. A. Wang
*** Date: May 26, 1993.
*** Copyright 1993, Massachusetts Institute of Technology, All Rights Reserved.
***
*** Description: header for shell commands
***************************************************************/

#ifndef MOTION_SH_INCLUDED
#define MOTION_SH_INCLUDED

static  char *copyright_msh="\
\n\n***********************************************\n\
File: motion-sh.h\n\
Author: John Y. A. Wang.\n\
The MIT Media Lab.\n\
Copyright (c) 1993, All Rights Reserved.\n\
***********************************************\n\n";

#include "motion.h"

#ifdef SVR4 
#include <string.h>
#else
#include <strings.h>
#endif


struct Affine_Analysis {
  int start;                /* start of analysis */
  int end;                  /* end of analysis */
  int tstep;                /* time step */
  int xstart;               /* crop region */
  int xend;                 /* crop region */
  int ystart;               /* crop region */
  int yend;                 /* crop region */
  int level;                /* level of coarse to fine */
  float method[10];         /* iteration method */
  float factor[6];     /* analysis factor mask */
  float param_init[6];      /* analysis factor mask */
};
typedef struct Affine_Analysis Affine_Analysis;

/************************/
struct Optical_Analysis {
  int start;                /* start of analysis */
  int end;                  /* end of analysis */
  int tstep;                /* time step */
  int level;                /* level of coarse to fine */
  int blur;                 /* blur level */
  float method[10];         /* iteration method */
  float noise;              /* noise */
  float xbias;              /* xbias */
  float ybias;              /* ybias */
  float intensity_bias;     /* intensity offset bias */
  float clipmin;
  float clipmax;
  float sum_filt[10];       /* summation filter */
  int sfdim;                /* summation filter dimensions */
  float pyr_filt[10];       /* pyramid  filter */
  int pfdim;                /* pyramid  filter dimensions */
  float exp_filt[10];       /* pyramid upsample interpolation filter */
  int expfdim;              /* pyramid upsample interpolation filter dimensions */
  int stat;                 /* flag to indicate to do statistics */  
};
typedef struct Optical_Analysis Optical_Analysis;
  
struct RLE_Display {
  int show;
  float gamma;
  float scale;
  float clipmin;
  float clipmax;
};
typedef struct RLE_Display RLE_Display;

struct Affine_Segment {
  int xdim;
  int ydim;
  int pdim;
  int numpatch;
  float ta;
  float tb;
  float tc;
  float terr;
  float tnear;
  int flag;
  int print;
  int n_iter;
  int n_iter2;
  float lambda;
};
typedef struct Affine_Segment Affine_Segment;

/********************************* fileio-sh.c ******************************/
FILE *set_in_file(/*ifp,infile*/);
FILE *set_out_file(/*ofp,outfile*/);
void free_images(/*malloc_images*/);
void close_files(/*opened_files*/);

/* allocates memory for an image sequence. consists of a linear array
 * of pointers to arrays. */
float **malloc_float_seq(/*nframes,imsize*/);

/* allocates memory for an image sequence. consists of a linear array
 * of pointers to arrays. */
unsigned char **malloc_uchar_seq(/*nframes,imsize*/);

/*********************************************/  
/* displays an image float array */
int display_float_image(/*image,xdim,ydim,title*/);

/* displays an image  byte array */
int display_array(/*array,xdim,ydim,title*/);

/* displays and image sequence of unsiged chars specified by an
** array of image pointers */
int display_seq_char(/*array,xdim,ydim,start,end,title*/);

/** returns a pointer to the token after skiping over n tokens.
 ** n==0 gives the nearest token. n==1 skip over 1 token ***/
char *strntok(/*cptr,n*/);

/*** fills the (int) array with scan tokens and returns the number of
 ***  tokens converted to (int) from string  ***/
int scan_str_int(/*str,array,max*/);

/*** fills the (float) array with scan tokens and returns the number of
 ***  tokens converted to (float) from string  ***/
int scan_str_float(/*str,float_array,max*/);


/****************************************************************************
 ** begin scaning arguments with the BEGIN_SCAN macro
 ** this wraps the looping around the scan args macros.
 ** Finish off the scaning with a END_SCAN
 */
#define BEGIN_SCAN(argc,argv,i,j,cmd) \
  for((i)=0,(j)=(i)-1;((i)<(argc))&&((j)!=(i));) { \
    (j)=(i); \
    if(((i)<(argc))&&(strcmp((argv)[i],"-help")==0)) { \
      fprintf(stderr,"\n\n%s -copyright\n\n",(cmd));return; }\
    if(((i)<(argc))&&(strcmp((argv)[i],"-copyright")==0)) {\
      fprintf(stderr,"\n%s: %s\n\n",argv[0],copyright);return; }


/* scan ith argument from command line */
/***** * scanarg1(argc,argv,i,"-xdim","%f",&xdim,-1); ****/
#define scanarg1(argc,argv,i,name,format,ptr,default) \
 if((i)==0) {(*(ptr))=(default);} \
 if(((i)<(argc))&&(strcmp((argv)[(i)],(name))==0)) {\
   if((((i)+1)>=(argc)) || (sscanf((argv)[((i)+1)],(format),(ptr))!=1)) {\
     fprintf(stderr,"\nError: \"%s %s\" argument has incorrect value.\n\n",\
       (argv)[(i)],((((i)+1)<(argc))?((argv)[((i)+1)]):("<missing>")));return;}\
   (i)+=2;}

/* set pointer to (i+1)th argument from command line */
/***** * setarg1(argc,argv,i,"-i",infile,NULL); ****/
#define setarg1(argc,argv,i,name,ptr,default) \
 if((i)==0) {(ptr)=(default);} \
 if(((i)<(argc))&&(strcmp((argv)[(i)],(name))==0)) {\
   if((((i)+1)>=(argc))) {\
     fprintf(stderr,"\nError: \"%s <missing>\" argument has no matching value.\n\n",\
       (argv)[(i)]);return;}\
   ptr=(argv)[((i)+1)];(i)+=2;}

/* set flag on ith arg from command line */
/***** * setflag1(argc,argv,i,"-pass",pass,1,0); ****/
#define setflag1(argc,argv,i,name,flag,setval,default) \
   if((i)==0) {(flag)=(default);} \
   if(((i)<(argc))&&(strcmp((argv)[(i)],(name))==0)) {\
        (flag)=(setval);(i)+=1;}

/* handle unsupported flags */
#define scan_unsupported(arg,cmd) \
  { fprintf(stderr,"\nError: unsupported flag %s\n\n",(arg)); \
    fprintf(stderr,"\n%s\n\n",(cmd));fflush(stderr); }

/*** This macro ends the scaning. Closes the loop and aslo
  ** does the error handling.
  **/
#define  END_SCAN(argc,argv,i,j,cmd) \
  if((i)==0) (i)++; \
  }\
  if((i)!=(argc)) { \
    scan_unsupported((argv)[i],(cmd)); return; \
  } 

/* set up the allocation for easier exiting and error handling */ 
#define MAXMALLOC 10
#define MAXNFILES 10


/****** For SUN and HP compatibility when calling fseek() **********/
#ifndef SEEK_SET
#define SEEK_SET 0
#endif
#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif
#ifndef SEEK_END
#define SEEK_END 2
#endif


#endif
