/*
 NAME
        api-defines.h
 DESCRIPTION
        This file contains many of the commonly used constant defines 
	for the automatic face recognition API.
 AUTHOR
        Mark Shackleton
 DATE
        20 June 1995

 COPYRIGHT BRITISH TELECOM PLC

 MODIFICATIONS:

*/

/*---------------------------------------------------------------------*/

/* only include this file once */
#ifndef __AFR_DEFS__
#define __AFR_DEFS__

/*** OPERATION CODES ***/

/* General Ops */

#define AFR_GEN_RESET                 1    /* reset pipeline: initialise */
#define AFR_GEN_MACRO_OP              2    /* macro: multiple ops given */

/* Face location related Ops */

#define AFR_LOC_FIND_COARSE          21    /* coarse full face location */
#define AFR_LOC_FIND_FINE            22    /* fine feature-based location */

/* Face recognition related Ops */

#define AFR_REC_CALC_COEFFS          41    /* calc face appearance coeffs */
#define AFR_REC_VERIFY_COEFFS        43    /* match face coeffs against a */
                                           /* single entry (verify)       */

/* Face database related Ops */

#define AFR_DB_GET_AVAIL_DBS         61    /* return list of available DBs */
#define AFR_DB_SET_NEW_DB            62    /* select current database to use */
#define AFR_DB_GET_ID_ENTRY          63    /* retrieve entry for given ID */
#define AFR_DB_GET_MATCH_ENTRIES     64    /* get list of entries matching */
                                           /* specific search criteria */
#define AFR_DB_ADD_ID_ENTRY          66    /* add new entry with given data */
#define AFR_DB_REPLACE_ID_ENTRY      67    /* replace specified entry in DB */
#define AFR_DB_DELETE_ID_ENTRY       68    /* delete specified entry from DB */

/*** RETURN CODES ***/

#define OP_SUCCESS                    0    /* operation succeeded */
#define OP_FAIL                      -1    /* error: see text error message */

/*** CONSTANT DEFINES ***/

#define DB_DIR           "DBS"
#define DB_DESCRIP_FILE  "descrip"

#define MAX_NUM_OPS                  10    /* max operations in a macro */
#define MAX_NUM_DBS                   5    /* max number of different DBs */
#define MAX_DB_IDS                  500    /* max number of people in DB */
#define GROW_SPARE_ID_SLOTS          10    /* add this many ID slots at once */
#define MAX_ENTRIES_PER_ID           10    /* max DB entries for one person */
#define MAX_FNAME_LEN               120    /* max length of a filename */
#define MAX_LINE_LEN                120    /* max generic i/p line length */
#define MAX_DB_NAME_LEN             120    /* max length of a DB's desciption */
#define FACE_LEFT_EYE                 0    /* indices... */
#define FACE_RIGHT_EYE                1
#define FACE_NOSE                     2
#define FACE_MOUTH                    3
#define NUM_FACIAL_FEATURES           4    /* left/right eye, nose, mouth */
#define MAX_NAME_LEN                120    /* max length of a person's name */
#define MAX_ALIASES                  20    /* max number aliases for a person */
#define MAX_DESCRIP_LEN             400    /* max free text description len */
#define MAX_ANNOTATIONS              20    /* max annotations per person */
#define MAX_ANNOTATE_LEN            120    /* max length of a one annotation */
#define ID_LEN                        3    /* zero padded field of this length */
#define ENTRY_LEN                     3    /* zero padded field of this length */

#define DB_NEXT                      -2    /* next sequential entry / id */
#define DB_RANDOM                    -3    /* randomly chose entry / id */
#define DB_NEW                       -4    /* refers to unique new id */
#define DB_ANY                       -5    /* any entry or id */

#define RAW_GREY_IMAGE                0    /* image types */
#define YUV_IMAGE                     1
#define RGB_IMAGE                     2
#define JPG_IMAGE                     3
#define YUV_INTERLEAVED_IMAGE         4
#define RGB_INTERLEAVED_IMAGE         5

/*** SEARCH PARAMETERS ***/
#define SEARCH_MIN_SCALE            0.7
#define SEARCH_MAX_SCALE            1.9
#define SEARCH_NUM_SCALES            21
#define MAX_NUM_COEFFS              100

/*** STANDARD DEFINES ***/

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE (!FALSE)
#endif

#ifndef False
#define False 0
#endif
#ifndef True
#define True (!False)
#endif

/*** FUNCTIONAL DEFINES ***/
#define rnd(x)		( (int)( (x)+0.5 ) )

#endif  /* ifndef __AFR_DEFS__ */

/*---------------------------------------------------------------------*/

