/*
 NAME
        api-structs.h
 DESCRIPTION
        This file contains many of the commonly data structure type definitions 
	of use to the automatic	face recognition API.
 AUTHOR
        Mark Shackleton
 DATE
        21 June 1995

 COPYRIGHT BRITISH TELECOM PLC

 MODIFICATIONS:

*/

/*----------------------------------------------------------------------------*/

/* only include this file once */
#ifndef __AFR_STRUCTS__
#define __AFR_STRUCTS__

/*----------------------------------------------------------------------------*/

/* some useful stuff */
/*
typedef int             boolean;
*/
typedef unsigned char   uchar;
/*
typedef unsigned short  ushort;
typedef unsigned int    uint;
*/

/*----------------------------------------------------------------------------*/

typedef struct s_operation_info {    /* description of an operation */

  int                op_code;        /* operation to be performed */
  void               *pparams;       /* ptr to operation parameters */

} operation_info_t;

/*----------------------------------------------------------------------------*/

typedef struct s_db_details {        /* details of particular database */

  char               db_name[ MAX_DB_NAME_LEN ];  /* DB name string */
  char               fname[ MAX_FNAME_LEN ];          /* DB filename */
  int                num_ids;        /*  number of people in DB */
  int                total_entries;  /*  total number of entries (records) */

} db_details_t;

/*----------------------------------------------------------------------------*/

typedef struct s_db_list {           /* list of available DBs */

  int                num_dbs;        /* total number of DBs available */
  db_details_t       db_details[ MAX_NUM_DBS ];  /* DB details */

} db_list_t;

/*----------------------------------------------------------------------------*/

typedef struct s_alias {      /* alias string */

  char               alias_str[ MAX_NAME_LEN ];

} alias_t;

/*----------------------------------------------------------------------------*/

typedef struct s_annotation {      /* annotation string */

  char               annotation_str[ MAX_ANNOTATE_LEN ];

} annotation_t;

/*----------------------------------------------------------------------------*/

typedef struct s_face_details {      /* description of a face database entry */

  int                id_num;
  int                entry_num;
  int                total_num_entries;
  char               name[ MAX_NAME_LEN ];
  alias_t            alias[ MAX_ALIASES ];
  char               description[ MAX_DESCRIP_LEN ];
  annotation_t       annotation[ MAX_ANNOTATIONS ];
  char               pathname[ MAX_FNAME_LEN ];  /* full entry filename */
  char               image_name[ MAX_FNAME_LEN ]; /* local image filename */
  float              coeffs[ MAX_NUM_COEFFS ];
  int                num_coeffs;
  char               coef_name[ MAX_FNAME_LEN ]; /* local coeffs filename */

} face_details_t;

/*----------------------------------------------------------------------------*/

typedef struct s_rectangle {       /* defines a rectangular region size */

  int                width;
  int                height;

} rectangle_t;

/*----------------------------------------------------------------------------*/

typedef struct s_coord {       /* defines a rectangular region size */

  int                x;
  int                y;

} coord_t;

/*----------------------------------------------------------------------------*/

typedef enum {                       /* status of position info */

  POS_UNKNOWN,                       /* position info is NOT known */
  POS_KNOWN,                         /* position info IS known */
  POS_HINT                           /* position info supplied as a hint */

} position_status_t;

/*----------------------------------------------------------------------------*/

typedef struct s_face_position {     /* face position info (inc. features) */

  coord_t            head_pos;
  float              head_scale;
  coord_t            feature_pos[ NUM_FACIAL_FEATURES ];
  position_status_t  head_status;
  position_status_t  feature_status[ NUM_FACIAL_FEATURES ];

} face_position_t;

/*----------------------------------------------------------------------------*/

typedef struct s_face_image {        /* image containing face */

  int                format;         /* image format type */
  int                xres;           /* resolution... */
  int                yres;
  float              pixel_aspect;   /* aspect ration of pels x/y */
  face_position_t    position;       /* details on face position */
  boolean            is_warped;      /* image has been warped */
  boolean            is_cropped;     /* image has been cropped */
  uchar              *data;          /* image pixel data */

} face_image_t;

/*----------------------------------------------------------------------------*/

typedef struct s_match_query {       /* info for DB query */
  char               name[ MAX_NAME_LEN ];
  alias_t            alias[ MAX_ALIASES ];
  char               description[ MAX_DESCRIP_LEN ];

  int                num_reqd;    /* number of matches required (max) */
                                      /* match these face coefficients */
  float              coeffs[ MAX_NUM_COEFFS ]; 
  int                num_coeffs;  /* match this many coeffs */
                                      /* match this part of a name/alias */
  char               name_substr[ MAX_NAME_LEN ];
                                      /* match this part of a description */
  char               descrip_substr[ MAX_DESCRIP_LEN ]; 
  annotation_t       annotation[ MAX_ANNOTATIONS ];

} match_query_t;

/*----------------------------------------------------------------------------*/

#endif  /* ifndef __AFR_STRUCTS__ */

/*----------------------------------------------------------------------------*/
