/*---------------------------------------------------------------------------

  Program Name:  best_minimum.c, check_minimum
  Purpose:       Best minimum is used to check if a given low point is indeed
                 a minimum when compared with its surrounding 8
		 neighbors.

		 Check_minimum is a boolean function that returns 1 if
		 a point is a minimum (compared to its eight
		 neighbors).  Best minimum just goes through the
		 appropriate points and then calls check_minimum for them.

  Author:        David Becker (dbecker@media.mit.edu) 
                 Baback Moghaddam (baback@media.mit.edu)

  Date:          Dec. 17, 1994

-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <float.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

void best_minimum(int imin, int imax, int jmin, int jmax,
		  int *rowm, int *colm, float *error,
		  float **errormap, float scale)

{
  float fval, fval1;
  int   i,j;

  fval = FLT_MAX;

  /* set default here in case of null windows or similar */
  i = imin+1;
  j = jmin+1;
  *rowm = (int) (i/scale+0.5);
  *colm = (int) (j/scale+0.5);
  *error = errormap[i][j];

  for (i=imin+1; i<=imax-1; i++) 
    {
      for (j=jmin+1; j<=jmax-1; j++) 
	{
	  fval1 = errormap[i][j];
	  if (fval1<fval) 
	    {
	      if (check_minimum(errormap,i,j))
		{
		  *rowm = (int) (i/scale+0.5);
		  *colm = (int) (j/scale+0.5);
		  *error = fval1;
		  fval = fval1;
		}
	    }
	}
    }
} 
      

int check_minimum(float **errormap, int i, int j)

{

  float n1,n2,n3,n4,n5,n6,n7,n8,n9;

  n1 = errormap[i-1][j-1];
  n2 = errormap[i-1][j];
  n3 = errormap[i-1][j+1];
  n4 = errormap[i][j-1];
  n5 = errormap[i][j];
  n6 = errormap[i][j+1];
  n7 = errormap[i+1][j-1];
  n8 = errormap[i+1][j];
  n9 = errormap[i+1][j+1];
  if (n5<n1 & n5<n2 & n5<n3 & n5<n4 & 
      n5<n6 & n5<n7 & n5<n8 & n5<n9) 
    return TRUE;

  return FALSE;

}
