




/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/******************************************************************/
/*
** File: 
**  
**   xvt4.c --- Xv test program 4
**   CORRECTED VERSION FOR HP VIDEO LIVE
**   Use with -depth 24 command line argument on a 24 bit screen
**
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**
** Revisions:
**   Martin Szummer, MIT Media Lab
**   05.15.91 Carver
**     - version 2.0 upgrade
**
**   18.04.91 Carver
**     - origional
**       
*/



#include "grab.h"

#define Xv_Xoffset 8
#define Xv_Yoffset 8


#define STOPVIDEO {\
  fprintf(stderr,"\n        StopVideo\n");\
  XvStopVideo(_Xv_Video_Dpy, _Xv_Port, _Xv_Main_Win);\
  \
  fprintf(stderr,"\n        Waiting for video Stopped event ...\n");\
  while (1)\
    {\
      XNextEvent(_Xv_Video_Dpy, &event);\
      if (event.type == event_base)\
	{\
	  pe = (XvEvent *)&event;\
	  if (pe->xvvideo.reason == XvStopped)\
	    {\
	      fprintf(stderr,"          Video stopped\n");\
	      break;\
	    }\
	  if (pe->xvvideo.reason == XvPreempted)\
	    {\
	      fprintf(stderr,"          Video preempted\n");\
	      break;\
	    }\
	  if (pe->xvvideo.reason == XvHardError)\
	    {\
	      fprintf(stderr,"          Video hard error\n");\
	      break;\
	    }\
	}\
    }\
}\


#define ABORT_MSG  fprintf(stderr,"\n  grabber_init() aborted\n")

int grabber_init(int argc, char **argv, int xdim, int ydim)
{
  int ii, status;
  int screen;

  Visual *def_vis;
  unsigned long vis_id;
  XVisualInfo *p_vis_info, vis_info_tmpl;
  XGCValues gc_attr;
  Colormap cmap;
  XColor scolor,ecolor;
  XSetWindowAttributes win_attr;
  XEvent event;

  unsigned long version, revision;
  unsigned int major_opcode;
  unsigned int error_base;
  unsigned long nAdaptors;
  XvAdaptorInfo *pAdaptors;

  unsigned int event_base;

  _Xv_Video_Dpy = XOpenDisplay(0);
  if (!_Xv_Video_Dpy) {
    fprintf(stderr,"\n  Couldn't open display\n");
    ABORT_MSG; return(-1);
  }
  root = XDefaultRootWindow(_Xv_Video_Dpy);
  screen = XDefaultScreen(_Xv_Video_Dpy);

  XSynchronize(_Xv_Video_Dpy, True);

  status = XvQueryExtension(_Xv_Video_Dpy, &version, &revision,
			    &major_opcode, &event_base, &error_base);

  if (status != Success) {
    fprintf(stderr,"\n  Xv video extension not available\n");
    ABORT_MSG; return(-1);
  }

  fprintf(stderr,"\n Using Xv V%01d.%d\n", version, revision);

  XvQueryAdaptors(_Xv_Video_Dpy, root, &nAdaptors, &pAdaptors);

  if (!nAdaptors)  {
    fprintf(stderr,"\n  Your display has no video adaptors\n");
    ABORT_MSG; return(-1);
  }

  if (!Setup(argc, argv, nAdaptors, pAdaptors, 
	     &_Xv_Port, &_Xv_Video_Depth, &vis_id)) {
    fprintf(stderr,"\n  Setup failed.\n");
    ABORT_MSG; return(-1);
  }

  vis_info_tmpl.visualid = vis_id;
  p_vis_info = XGetVisualInfo(_Xv_Video_Dpy, VisualIDMask, 
			      &vis_info_tmpl, &ii);
  if (!p_vis_info) {
    fprintf(stderr,"      Error: Couldn't find visual ");
    fprintf(stderr,"#%x listed for adaptor.\n", vis_id);
    ABORT_MSG; return(-1);
  }

  _Xv_Visual = p_vis_info->visual;
  def_vis = XDefaultVisual(_Xv_Video_Dpy,screen);

  if (_Xv_Visual->visualid == def_vis->visualid)
    cmap = XDefaultColormap(_Xv_Video_Dpy,screen);
  else
    cmap = XCreateColormap(_Xv_Video_Dpy, root, _Xv_Visual, AllocNone);

/* "midnight blue" causes the darkest regions to be midnight blue??? */
/*  XAllocNamedColor(_Xv_Video_Dpy, cmap, "midnight blue", &scolor, &ecolor); */

  XAllocNamedColor(_Xv_Video_Dpy, cmap, "black", &scolor, &ecolor);

  win_attr.colormap = cmap;
  win_attr.background_pixel = scolor.pixel;
  win_attr.event_mask = ExposureMask | VisibilityChangeMask | 
    ButtonPressMask | KeyPressMask;
  win_attr.backing_store = Always;
  win_attr.border_pixel = scolor.pixel;

  _Xv_Main_Win = XCreateWindow(_Xv_Video_Dpy, root, 0, 0, xdim+16, 2*ydim+8, 0, 
			   _Xv_Video_Depth, InputOutput, _Xv_Visual,
			   CWColormap | CWBackPixel | CWEventMask |
			   CWBackingStore | CWBorderPixel,
			   &win_attr);

  XMapWindow(_Xv_Video_Dpy, _Xv_Main_Win);

  fprintf(stderr,"\n\n  Waiting for window to become visible...\n\n");
  while (1) {
    XNextEvent(_Xv_Video_Dpy, &event);
    if (event.type == VisibilityNotify) break;
  }

/*  fprintf(stderr,"\n  SelectVideoNotify on window\n"); */
  XvSelectVideoNotify(_Xv_Video_Dpy, _Xv_Main_Win, True);
  
  gc_attr.foreground = scolor.pixel;
  _Xv_Video_GC = XCreateGC(_Xv_Video_Dpy, _Xv_Main_Win, GCForeground, &gc_attr);

/*  fprintf(stderr,"\n  PutVideo\n\n");*/
  XvPutVideo(_Xv_Video_Dpy, _Xv_Port, _Xv_Main_Win, _Xv_Video_GC,
	     0, 0, 640, 480, Xv_Xoffset, 0, xdim, ydim);

  fprintf(stderr,"  grabber_init() successful.\n");
  return(0);
}


Setup(argc, argv, nAdaptors, pAdaptors, p_port, p_depth, p_vis_id)
     int         argc;
     char        *argv[];
     unsigned long nAdaptors;
     XvAdaptorInfo *pAdaptors;
     unsigned long *p_port;
     unsigned long *p_depth;
     unsigned long *p_vis_id;
{
  int ii;
  XvAdaptorInfo *pAdaptor;
  XvFormat *pFormat;
  int adaptor, port, depth, visual_id;

  adaptor = port = depth = visual_id = -1;

  /* LOOK THROUGH COMMAND LINE ARGUMENTS */

  for ( ii = 1; ii < argc; ii++ )
    {
      if (strcmp( argv[ii], "-adaptor") == 0)
	{
	    if(++ii < argc)
	      adaptor = atoi(argv[ii]);
            else
	      UseMsg(argv);
	}
      else if (strcmp( argv[ii], "-port") == 0)
	{
	    if(++ii < argc)
	      port = atoi(argv[ii]);
            else
	      UseMsg(argv);
	}
      else if (strcmp( argv[ii], "-depth") == 0)
	{
	    if(++ii < argc)
	      depth = atoi(argv[ii]);
            else
	      UseMsg(argv);
	}
      else if (strcmp( argv[ii], "-visual") == 0)
	{
	    if(++ii < argc)
	      visual_id = atoi(argv[ii]);
            else
	      UseMsg(argv);
	}
      else if (strcmp( argv[ii], "-help") == 0)
	{
	  UseMsg(argv);
	}
    }

  if (adaptor < 0)
    {
      pAdaptor = pAdaptors;
    }
  else
    {
      if (adaptor > nAdaptors)
	{
	  fprintf(stderr,"\n  Adaptor #%d doesn't exist.\n", adaptor);
	  return False;
	}
      pAdaptor = pAdaptors+(adaptor-1);
    }

  if (port < 0)
    {
      port = pAdaptor->base_id;
    }
  else
    {
      if (port > pAdaptor->num_ports)
	{
	  fprintf(stderr,"\n  Port #%d doesn't exist for adaptor #%d.\n", 
		 port, adaptor);
	  return False;
	}
    }

  pFormat = pAdaptor->formats;

  if (depth < 0)
    {
      depth = pFormat->depth;
    }
  else
    {
      for (ii=0; ii<pAdaptor->num_formats; ii++)
	{
	  if (pFormat->depth == depth) break;
	  pFormat++;
	}

      if (ii >= pAdaptor->num_formats)
	{
	  fprintf(stderr,"\n  Depth %d not supported by adaptor.\n", depth);
	  return False;
	}
    }

  if (visual_id < 0)
    {
      visual_id = pFormat->visual_id;
    }
  else
    {

      pFormat = pAdaptor->formats;

      for (ii=0; ii<pAdaptor->num_formats; ii++)
	{
	  if ((pFormat->visual_id == visual_id) && pFormat->depth == depth)
	    break;
	  pFormat++;
	}

      if (ii >= pAdaptor->num_formats)
	{
	  fprintf(stderr,"\n  Visual-id %d at depth %d not supported by adaptor.\n", 
		 depth, visual_id);
	  return False;
	}
    }

  *p_port = port;
  *p_depth = depth;
  *p_vis_id = visual_id;

  return True;
}

UseMsg(argv)
     char        *argv[];
{

  fprintf(stderr,"use: %s [option]\n", argv[0]);
  fprintf(stderr,"\t-adaptor #                    adaptor number\n");
  fprintf(stderr,"\t-port #                       port (XID)\n");
  fprintf(stderr,"\t-depth #                      drawable depth (planes)\n");
  fprintf(stderr,"\t-visual #                     drawable visual (id)\n");
  exit();
}


/**** get from video and put in float or byte array ***/
int get_image_video(void *im, int xdim, int ydim, char *type, int interleave)
{
  XImage *new_image;
  int i,j, imsize=xdim*ydim;
  unsigned char *cp;

  XvPutVideo(_Xv_Video_Dpy, _Xv_Port, _Xv_Main_Win, _Xv_Video_GC,
	     0, 0, 640, 480, Xv_Xoffset, Xv_Yoffset, xdim, ydim);

  new_image = XGetImage(_Xv_Video_Dpy, _Xv_Main_Win, Xv_Xoffset, Xv_Yoffset,
			xdim, ydim, -1, ZPixmap);

  if(strcmp(type,"float")==0) {
    float *rp_f,*gp_f,*bp_f;
    float *im_f=(float *)im;

    if(interleave==1) {
/*      fprintf(stderr,"float interleaved\n"); */
      cp=(unsigned char *)new_image->data;
      for(i=0,j=0;i<imsize;i+=3,j+=4) {
	im_f[i+0]=(float)cp[j+1];
	im_f[i+1]=(float)cp[j+2];
	im_f[i+2]=(float)cp[j+3];
      }
    }
    else {
/*      fprintf(stderr,"float non-interleaved\n"); */
      rp_f=im_f; gp_f=im_f+imsize; bp_f=im_f+2*imsize;
      cp=(unsigned char *)new_image->data;
      for(i=0,j=0;i<imsize;i++,j+=4) {
	rp_f[i]=(float)cp[j+1];
	gp_f[i]=(float)cp[j+2];
	bp_f[i]=(float)cp[j+3];
      }
    }
  }
  else if(strcmp(type,"uchar")==0) {
    unsigned char *rp_b,*gp_b,*bp_b;
    unsigned char *im_b=(unsigned char*) im;

    if(interleave==1) {
/*      fprintf(stderr,"uchar interleaved\n"); */
      cp=(unsigned char *)new_image->data;
      for(i=0,j=0;i<imsize;i+=3,j+=4) {
	im_b[i+0]=(unsigned char)cp[j+1];
	im_b[i+1]=(unsigned char)cp[j+2];
	im_b[i+2]=(unsigned char)cp[j+3];
      }
    }
    else {
/*      fprintf(stderr,"uchar non-interleaved\n"); */
      rp_b=im_b; gp_b=im_b+imsize; bp_b=im_b+2*imsize;
      cp=(unsigned char *)new_image->data;
      for(i=0,j=0;i<imsize;i++,j+=4) {
	rp_b[i]=(unsigned char)cp[j+1];	gp_b[i]=(unsigned char)cp[j+2];
	bp_b[i]=(unsigned char)cp[j+3];
      }
    }
  }
  else { fprintf(stderr,"unsupported type \"%s\"\n",type); return(0); }

  new_image->depth=24;
  XPutImage(_Xv_Video_Dpy, _Xv_Main_Win, _Xv_Video_GC,
	    new_image, 0, 0, Xv_Xoffset, ydim+Xv_Yoffset, xdim, ydim);
  
  XDestroyImage(new_image);

  return(0);
}

/**** get from video and put in float array ***/
int get_image_video_f(float *im, int xdim,int ydim,int interlace)
{
  return(get_image_video((void *)im, xdim, ydim, "float",interlace));
}

/**** get from video and put in unsigned char array ***/
int get_image_video_uc(unsigned char *im, int xdim,int ydim,int interlace)
{
  return(get_image_video((void *)im, xdim, ydim, "uchar",interlace));
}

/**************** rgb to y conversion ***********************/
int rgb2y(float *r, float *g, float *b, float *y,
	  int xdim, int ydim)
{
  int k,imsize=xdim*ydim;

  for(k=0;k<imsize;k++) {
    *y = (0.299*(*r) + 0.587*(*g) + 0.114*(*b));
    y++; r++; g++; b++;
  }
}


int put_image_video(float *im, int xdim, int ydim, Window win)
{
  XImage *new_image;
  int i,imsize=xdim*ydim;
  float *rp,*gp,*bp;
  unsigned char *cp;
  char *data;

  data=(char*)malloc(sizeof(unsigned long)*imsize);

  for(i=0,rp=im,gp=im+imsize,bp=im+2*imsize,
      cp=(unsigned char *)data;
      i<imsize;i++) {
    *cp=(unsigned char)0; cp++;
    *cp=F2UC(*rp); rp++; cp++;
    *cp=F2UC(*gp); gp++; cp++;
    *cp=F2UC(*bp); bp++; cp++;
  }

  new_image = XCreateImage(_Xv_Video_Dpy, _Xv_Visual, 24,ZPixmap,0,
			   data,xdim,ydim,32,0);
  new_image->depth=24;

  XPutImage(_Xv_Video_Dpy, win, _Xv_Video_GC, new_image,
	    0, 0, 0, ydim, xdim, ydim);

  XDestroyImage(new_image);

  return(0);
}



/*********************************************************************************/
/*********** XvPutStill compatibility **************************/
#if 0
void XvPutStilljw(Display *_Xv_Video_Dpy, XvPortID port, Window win, GC gc,
		  int src_x, int src_y, int src_xdim, int src_ydim,
		  int dest_x, int dest_y, int dest_xdim, int dest_ydim)
{
  XImage *image;
  XvPutVideo(_Xv_Video_Dpy,port,win,gc,src_x,src_y,src_xdim,src_ydim,
	     src_x,src_y,dest_xdim,dest_ydim);

  image = XGetImage(_Xv_Video_Dpy, _Xv_Main_Win,src_x, src_y, dest_xdim, dest_ydim, 
		    -1, ZPixmap);

  image->depth=24;
  XPutImage(_Xv_Video_Dpy, _Xv_Main_Win, gc, image, 0, 0, dest_x, dest_y,
	    dest_xdim, dest_ydim);

  XDestroyImage(image);

  XvPutVideo(_Xv_Video_Dpy,port,win,gc,src_x,src_y,src_xdim,src_ydim,
	     src_x,src_y,src_xdim,src_ydim);
  
}
#endif
