/*---------------------------------------------------------------------------

  Program Name:  load_eigen.c
  Purpose:       function for loading eigenvectors into memory.
                 
  Author:        Dave Becker
  Date:          Sept. 26, 1994

  ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <malloc.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

int load_eigen(char eigdir[MAX_CHARS], int neigframe_select, 
	       int neigframe_in, int *neigframe,
	       float ****eigimages, int *nrow, int *ncol)
     
{
  int sets, bytes_pixel;
  int f;
  char filename[MAX_CHARS];

  /* ----  read Eigenimage DAT-file parameters and eigenvectors -------- */
  
  read_descriptor(eigdir, neigframe, &sets, &bytes_pixel, ncol, nrow);
  if (sets>1) 
    myerror("Eigenvector files must be single-set DAT files!");
  if (neigframe_select==1) 
    {
      if (neigframe_in>*neigframe)
	myerror("Can't use more Eigenimages than available!");
      *neigframe = neigframe_in;
    }
  *eigimages = f3tensor(0, *neigframe-1, 1, *nrow, 1, *ncol);
  for (f=0; f<*neigframe; f++) 
    {
      if (bytes_pixel==1)
	read_DAT_frame((*eigimages)[f], eigdir, f, *neigframe, *nrow, *ncol);
      if (bytes_pixel==4) 
	{
	  sprintf(filename,"%s/data%d", eigdir, f);
	  read_RAW_float(filename, (*eigimages)[f], *nrow, *ncol);
	}
    }
  
  return 0;

}
