/*---------------------------------------------------------------------------

  Program Name:  load_known.c
  Purpose:       Loads all the known faces from a directory containing their
                 names and coefficients.

  Author:        David Becker, dbecker@media.mit.edu
  Date:          Sept. 30, 1994

-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <malloc.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

void load_known(Kface **kfaces, int *num_k_coeffs, int *num_k, 
	       char k_dir[MAX_CHARS])

{

  int   file_num = 0;
  FILE *fklist,*fknown;
  char  klist[MAX_CHARS];
  float dummy;
  char line[MAX_CHARS];
  char kfile[MAX_CHARS];
  int point;

/*** DEBUG ***/
fprintf(stderr,"Entering load_known()\n");


/*** DEBUG ***/
fprintf(stderr,"*num_k = %d\n",*num_k );


  /*  Looks for a list file in the known images directory */
  
  sprintf(klist,"%s/%s",k_dir,"list");
  *num_k = number_files(klist);

/*** DEBUG ***/
fprintf(stderr,"*num_k = %d\n",*num_k );


/*** DEBUG ***/
fprintf(stderr,"*num_k = %d\n",*num_k );


  /* Allocate space for coefficients and name for each face */

  *kfaces = (Kface *) calloc(*num_k, sizeof(Kface));
  if (*kfaces == NULL) 
    {
      fprintf(stderr, "\nERROR: Memory cannot be allocated\n\n");
      exit(3);
    }
  
  
  /*   Read in all the known data into the array of kfaces    */

  if ((fklist = fopen(klist, "r")) == NULL)
    {
      fprintf(stderr, "\nERROR: Input file %s cannot be opened\n\n", klist);
      exit(2);
    }
  
  /* Get number of coefficients in the files */

  file_num = 0;
  if (fscanf(fklist,"%s",line) == 0)
    {
      fprintf(stderr,"\nERROR in the known faces directory list\n\n");
      exit(2);
    }
  sprintf(kfile, "%s/%s", k_dir, line);
  if ((fknown = fopen(kfile,"r")) == NULL)
    {
      fprintf(stderr,"\nERROR: in opening %s\n\n",kfile);
      exit(2);
    }
  *num_k_coeffs = 0;
  while (fscanf(fknown,"%f",&dummy) != EOF)
    (*num_k_coeffs)++;
  fclose(fknown);
  rewind(fklist);

/*** DEBUG ***/
fprintf(stderr,"*num_k_coeffs = %d\n",*num_k_coeffs);


  /* Now actually load in coefficients and names */


  while ((fscanf(fklist, "%s", line)) != EOF)
    {
      if ((line[0] != '#') && (strlen(line) > 0))
	{
	  (*kfaces)[file_num].name = (char *) calloc((strlen(line) +
						      1), sizeof(char));
	  sprintf(((*kfaces)[file_num]).name,"%s",line);
	  (*kfaces)[file_num].coeffs = 
	    (float *) calloc(*num_k_coeffs + 1, sizeof(float));
	  sprintf(kfile, "%s/%s", k_dir, line);
	  (*kfaces)[file_num].coeffs[0] = 0;
	  point = 1;
	  if ((fknown = fopen(kfile, "r")) == NULL)
	    {
	      fprintf(stderr, "\nERROR: Input file %s cannot be opened", 
		      kfile);
	      exit(2);
	    }
	  while ((fscanf(fknown, "%f",&((*kfaces)[file_num].coeffs[point++])) 
		  != EOF));
	  file_num++;
	  fclose(fknown);
	}
    }
  fclose(fklist);

/*** DEBUG ***/
fprintf(stderr,"Leaving load_known()\n");
     
}
