/*---------------------------------------------------------------------------

  Program Name:  make_coeffs.c
  Purpose:       Given an input image eigenvectors, produces
                 coefficients by projection.

  Author:        Dave Becker, converting Baback Moghaddam's code
  Date:          August 29, 1994

-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <float.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

void make_coeffs(int in_ncol, int in_nrow, int ncol, int nrow, 
		int residual_select, int neigframe,
		float **image, float *coeffs, float ***eigimages)
{

  register int i, j, k;
  float        fval, **test, error;

  /* ----  check dimensions of files in question -------- */

  if (nrow!=in_nrow || ncol!=in_ncol) 
    {
      fprintf(stderr,"ERROR: Input dimensions");
      fprintf(stderr,
	      " %d-by-%d don't match eigenvector dimensions %d-by-%d.\n",
	      in_nrow, in_ncol, nrow, ncol);
      exit(1);
    }

  if (residual_select>0)
    test = matrix(1, nrow, 1, ncol);

  /* -------- compute projection coefficients ------ */
      
  for (i=1; i<=nrow; i++) 
    for (j=1; j<=ncol; j++) 
      image[i][j] -= eigimages[0][i][j];
      

  for (k=1; k<neigframe; k++) 
    {
      fval = 0.0;
      for (i=1; i<=nrow; i++)
	for (j=1; j<=ncol; j++) 
	  fval += (image[i][j] * eigimages[k][i][j]);
      coeffs[k] = fval;
    }

  /* --------- DEBUG:  reconstruction -------------- */

  if (residual_select>0) 
    {
      for (i=1; i<=nrow; i++) 
	for (j=1; j<=ncol; j++) 
	  {
	    fval = 0.0;
	    for (k=1; k<neigframe; k++) 
	      fval += (eigimages[k][i][j] * coeffs[k]);
	    test[i][j] = fval;
	  }
      error = 0.0; 
      for (i=1; i<=nrow; i++) 
	for (j=1; j<=ncol; j++) 
	  error += SQR(test[i][j] - image[i][j]);
      fprintf(stdout,"residual = %g  \n", error/(nrow*ncol));
    }

  return;
  

}







