/*----------------------------------------------------------------------
PROGRAM: scalewarp.c
DATE:    6/16/94
AUTHOR:  dbecker@media.mit.edu

This function, scalewarp, scales and translates an image so as to
place the (x_coord,y_coord) point at (x_dest,y_dest) (as defined in
defines.h.

------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <float.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

/*----------------------------------------------------------------------*/

void scale_warp(srchparams_t sparms, 
	   int x_coord,
	   int y_coord,
	   float fscale,
	   image_t scaledimage)
{

  register int i,j;
  int sets, bytes_pixel;
  int nrow, ncol, out_nrow, out_ncol;
  image_t temp_im;

  float **W; 
  
  float x_dest, y_dest;
  float scalefactor = SCALEFACTOR;
  float scale;

  float bgvalue = BGVALUE;

  x_dest = X_DEST;
  y_dest = Y_DEST;

  /* ----  read srchparams' descriptor file parameters -------- */
  sets = sparms.sets;
  bytes_pixel = sparms.bytes_pixel;

  nrow = sparms.nrow;
  ncol = sparms.ncol;
  /*
  nrow = MS_EIGSEARCH_NROW;
  ncol = MS_EIGSEARCH_NCOL;
  */
  out_nrow = SCALEWARP_NROW;
  out_ncol = SCALEWARP_NCOL;

  if (sets>1) 
    myerror("Input files must be single-set DAT files!");

  /* ---- set up matrices  ---- */

  W    = matrix(1, 3, 1, 3);

  /* --- reference coordinates --- */
      
  scale = fscale * scalefactor;
  for (i=1; i<=3; i++) 
    for (j=1; j<=3; j++)
      W[i][j] = 0.0;
  W[1][1] = W[2][2] = scale;
  W[3][3] = 1.0;
  W[1][3] = x_dest - scale*x_coord;
  W[2][3] = y_dest - scale*y_coord;

  /* --- apply warp to image --- */
  temp_im = matrix(1,nrow,1,ncol);

  i = affine_warp(sparms.orig_image,temp_im,nrow,ncol,W,bgvalue);

  /* MAS */
  if (out_nrow > nrow || out_ncol > ncol)
    fprintf(stderr,"scalewarp: WARNING: source image is too small!\n");

  for (i = 1; i <= out_nrow; i++)
    for (j = 1; j <= out_ncol; j++)
      scaledimage[i][j] = temp_im[i][j];

  /* --- clear used matrices --- */
      
  free_matrix(W, 1, 3, 1, 3);
  free_matrix(temp_im,1,nrow,1,ncol);
  return;
}

















