/*----------------------------------------------------------------------
PROGRAM: decimate.c
DATE:    5/5/94
AUTHOR:  Baback Moghaddam, baback@media.mit.edu

This function simply does subsampling on images.


------------------------------------------------------------------------


   

---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <float.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"


/*----------------------------------------------------------------------*/


void subsample(float **image_in, int nrow, int ncol, 
	       float **image_out, int *nrow_out, int *ncol_out,
	       float a_parameter)

     /* takes pre-allocatd input/output FLOAT image matrices, a GP-filter */
     /* a-parameter and does a factor-of-two subsampling on the input */
	
{
  register int i,j;
  int M, N;
  float *sep_filter;
  float **image_temp;   

  /* ---- setup separable 5-tap filters  ---- */

  sep_filter = vector(1, 5);  
  if (a_parameter>0) {
    sep_filter[1] = sep_filter[5] = 0.25 - a_parameter/2.0;
    sep_filter[2] = sep_filter[4] = 0.25;
    sep_filter[3] = a_parameter;
  }


  /* ----- compute output image dimensions & allocate temp image ------ */
  
  image_temp = matrix(1, nrow, 1, ncol);

  M = (int) nrow/2.0;
  N = (int) ncol/2.0;
     

  /* ------ filter the image ----------------- */

  if (a_parameter>0) 
    conv2d_sep(image_in, image_temp, nrow, ncol, 
	       sep_filter, 5, sep_filter, 5);
  else {
    for (i=1; i<=nrow; i++)
      for (j=1; j<=ncol; j++)
	image_temp[i][j] = image_in[i][j];
  }
  
      
  /* ------ subsample the image -------------- */
  
  for (i=1; i<=M; i++)
    for (j=1; j<=N; j++)
      image_out[i][j] = (image_temp[2*(i-1)+2][2*(j-1)+2] + 0.5);
  

  *nrow_out = M;
  *ncol_out = N;
 
  free_vector(sep_filter, 1, 5);
  free_matrix(image_temp, 1, nrow, 1, ncol);

}
