/***************************************************************
*** File: grab-video.c
*** Author: John Y. A. Wang
*** Date: July 15, 1994
*** Copyright 1994, Massachusetts Institute of Technology, All Rights Reserved.
***
*** Description: optic flow from command line with input/output pipes
***       input is byte. output is float: yflow1,xflow1,yflow2,xflow2....
***************************************************************/

static  char *copyright="\
\n\n***********************************************\n\
File: grab-video.c\n\
Author: John Y. A. Wang.\n\
The MIT Media Lab.\n\
Copyright 1994, All Rights Reserved.\n\
***********************************************\n\n";
#include "grab.h"
#include "motion-sh.h"
#include "defines.h"
#include "/mas/vision/users/tpminka/include/tpm/memcheck.h"

#define EXIT {\
    free_images(malloc_images,nmalloc); close_files(opened_files,nfiles);\
    return;}

/***************************************************************/
void video_init()
{
  int i,j;

  int xdim = 640,ydim = 480,imsize;

  unsigned char *im1;
  Window out_win;
  int gray;

  char *malloc_images[MAXMALLOC];
  int nmalloc=0;
  FILE *opened_files[MAXNFILES];
  int nfiles=0;
  int interleave=0;
  
  char *cmd="\
grab-test [-depth {24,8}] -xdim -ydim [-gray]";

  int video_argc=0, depth;
  char *video_argv[10], depthstr[3];
  Display *display;

#define GRAB_ALL 1
#define GRAB_ONE 2
#define GRAB_STOP 3

  XEvent event;    /* for notifying window mouse clicks */

  int grab_state=GRAB_STOP;

  /********** begin ***********/
  for(i=0;i<MAXMALLOC;i++) { malloc_images[i]=NULL; }
  for(i=0;i<MAXNFILES;i++) { opened_files[i]=NULL;  }

  /********************* begin scan input ************************/
/*  BEGIN_SCAN(argc,argv,i,j,cmd) {
    scanarg1(argc,argv,i,"-xdim","%d",&xdim,640);
    scanarg1(argc,argv,i,"-ydim","%d",&ydim,480);
    scanarg1(argc,argv,i,"-depth","%d",&depth,24);

    setflag1(argc,argv,i,"-gray",gray,1,0);
    setflag1(argc,argv,i,"-I",interleave,1,0);
  }
  END_SCAN(argc,argv,i,j,cmd);*/
  depth = 24;
  gray = 1;
  interleave = 1;

  /************************ end scan input ***************************/

  video_argc=0;
  video_argv[video_argc]="grab-test";
  video_argc++;

  video_argv[video_argc]="-depth"; video_argc++;
  video_argv[video_argc]=depthstr; sprintf(depthstr,"%2d",depth); video_argc++;

#if DBG
  fprintf(stderr,"video args: video_argc=%d\n\n\"",video_argc);
  for(i=0;i<video_argc;i++) fprintf(stderr,"%s ",video_argv[i]);
  fprintf(stderr,"\"\n\n");
#endif

  grabber_init(video_argc,video_argv,xdim,ydim);
}

void get_video_image(float *image, char filename[MAX_CHARS], int de_flag)
{
  int i,j;

  int xdim = 640,ydim = 480,imsize;

  float *im1;
  unsigned char *tempim;
  Window out_win;
  int gray;

  char *malloc_images[MAXMALLOC];
  int nmalloc=0;
  FILE *opened_files[MAXNFILES], *file_ptr;
  int nfiles=0;
  int interleave=0;
  char *cmd="\
grab-test [-depth {24,8}] -xdim -ydim [-gray]";

  int video_argc=0, depth;
  char *video_argv[10], depthstr[3];
  Display *display;

#define GRAB_ALL 1
#define GRAB_ONE 2
#define GRAB_STOP 3

  XEvent event;    /* for notifying window mouse clicks */

  int grab_state=GRAB_ONE;
  depth = 24;
  gray = 1;
  interleave = 0;

  video_argc=0;
  video_argv[video_argc]="grab-test";
  video_argc++;

  video_argv[video_argc]="-depth"; video_argc++;
  video_argv[video_argc]=depthstr; sprintf(depthstr,"%2d",depth); video_argc++;

#if DBG
  fprintf(stderr,"video args: video_argc=%d\n\n\"",video_argc);
  for(i=0;i<video_argc;i++) fprintf(stderr,"%s ",video_argv[i]);
  fprintf(stderr,"\"\n\n");
#endif

  imsize=xdim*ydim;
  im1 = (float *)malloc(sizeof(float)*3*imsize);
  tempim = (unsigned char *)malloc(sizeof(unsigned char)*3*imsize);
      
  display=_Xv_Video_Dpy;
  grab_state=GRAB_ONE;

  get_image_video(im1,xdim,ydim,"float",interleave);
  for (i = 0; i < imsize*3; i++)
      tempim[i] = (unsigned char)im1[i];

  /* If de_flag is set, write color image to disk */
  
  if (de_flag)
    {
      if ((file_ptr = fopen(filename,"w")) == NULL) 
	fprintf(stderr,"ERROR writing color image to disk\n");

      fwrite(tempim,sizeof(unsigned char),3*imsize,file_ptr);
      fclose(file_ptr);
    }

  free(tempim);

  if(gray==1) 
    {   /* convert RGB to  Y */
      rgb2y(im1,im1+imsize,im1+2*imsize,image,xdim,ydim);
    }
  else 
    {     /* write out RGB */
      fwrite(im1,sizeof(unsigned char),3*imsize,stdout);      
    }
  free(im1);
}


