/* This file contains functions that are used in function calls in the */
 /* file, pipe_neighbor.c.  Each function is documented below */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "defines.h"
#include "util.h"
#include "io.h"
#include "matrix.h"
#include "affine.h"
#include "convolve.h"
#include "functions.h"
#include "faces.h"

/* This simply finds the Euclidean distance between two vectors */

float find_distance(float *test, float *known, int no)
{
  int i;
  float sum=0;
  
  for (i=0; i<no; i++){
    sum += ( (test[i] - known[i]) * (test[i]-known[i]));
  }
  return (sqrt(sum));
}


/* This adds a node into a linked list in order of decreasing distance */


Matchptr add_match(Matchptr root, Matchptr new, int n)
{
  Matchptr temp, prev;
  int count = 0;

  if (root==NULL)
    return new;
  else {
    if (root->distance > new->distance){
      new->next = root;
      return new;
    }
  }
  
  temp = root;
  prev = root;
  while ( ((temp != NULL) && (count != n))  
	 && (temp->distance <= new->distance)){
    prev = temp;
    temp = temp->next;
    count++;
  }
  new->next = temp;
  prev->next = new;
  return root;
}


/* This simply returns the number of files included in a list file */

int number_files(char list[])
{
  int no_files;
  char line[MAX_CHARS];
  FILE *flist;
  
  
  /* Loop over input list file and calculate total no. of 
     test files and create the array of tfaces */
  if ( (flist = fopen(list, "r")) == NULL){
    fprintf(stderr, "\nERROR: Input file %s cannot be opened \n\n", list);
    exit(2);
  }
  no_files = 0;
  while (fscanf(flist, "%s", line) != EOF){
    if ( (strncmp(line, "#", 1) != 0) && (strlen(line) > 1))
      no_files++;
  }
  
  fclose(flist);
  return (no_files);
}







